@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # encoding: utf-8
 """
 CodeParser.py
@@ -122,7 +122,7 @@ def nonlocalDimensionAccessForVectors(vectors, codeBlock):
     componentNameToVectorMap = {}
     for v in vectors:
         componentNameToVectorMap.update(dict.fromkeys(v.components, v))
-    result = nonlocalDimensionAccessForComponents(componentNameToVectorMap.keys(), codeBlock)
+    result = nonlocalDimensionAccessForComponents(list(componentNameToVectorMap.keys()), codeBlock)
     return [(componentName, componentNameToVectorMap[componentName], resultDict, codeSlice) \
                 for componentName, resultDict, codeSlice in result]
 
@@ -175,9 +175,9 @@ def nonlocalDimensionAccessForComponents(components, codeBlock):
         # Addendum by Joe: this occasionally killed us on macOS.  Switching from 
         #   .iterItems() to .items() seems to fix it though. 
         
-        tokenDict = dict(tokens.access[0].items())
+        tokenDict = dict(list(tokens.access[0].items()))
 
-        for key, value in tokenDict.items():
+        for key, value in list(tokenDict.items()):
             accessDict[key] = (' '.join(value[0].asList()), value.slice.start)
         results.append((tokens.name, accessDict, slice(start, end)))
     return results
@@ -351,7 +351,13 @@ def performIPOperatorSanityCheck(componentName, propagationDimension, operatorCo
             if len(node) == 1: continue
             # idx is the index in the stack of the next element *deeper* in the parse tree.
             previousStackEntry = stack[idx]
-            if not isinstance(stack[idx], basestring):
+
+            # The following check breaks python 2 to python 3 compatibility, and just replacing basestring->str
+            # doesn't work, since this is a case where stack[idx] can be unicode. So use code that works for
+            # both python 2 and python 3.
+
+            # Old code: if not isinstance(stack[idx], str):
+            if not isinstance(stack[idx], ("".__class__, u"".__class__)):
                 previousStackEntry = previousStackEntry.asList()
             binaryOpIdx = node.asList().index(previousStackEntry) - 1
             if binaryOpIdx < 0: binaryOpIdx = 1
@@ -388,7 +394,7 @@ def performIPOperatorSanityCheck(componentName, propagationDimension, operatorCo
             stack.append(results.targetOperator)
             raise FoundTargetException()
         for item in results:
-            if isinstance(item, basestring): continue
+            if isinstance(item, str): continue
             findOperatorInParseTree(item)
         del stack[-1]
     
