@@ -149,6 +149,7 @@ class _HermiteGaussTransform(MMT):
       lattice = lattice,
       transformations = dict([
         ((name, 'n' + name), coordinate2SpectralBasisChange),
+        ((name + '_3f', 'n' + name), coordinate2SpectralBasisChange),
         ((name + '_4f', 'n' + name), coordinate2SpectralBasisChange),
         (('k' + name, 'n' + name + '_twiddle'), coordinate2SpectralBasisChange),
         (('n' + name, 'n' + name + '_twiddle'), spectralBasisTwiddleChange),
@@ -200,12 +201,20 @@ class _HermiteGaussTransform(MMT):
     )
     dim.addRepresentation(twiddleSpace)
     
+    threeFieldCoordinateSpace = HermiteGaussDimensionRepresentation(
+      name = name + '_3f', type = type, runtimeLattice = lattice, _maximum = maximum,
+      stepSizeArray = True, parent = dim, tag = self.auxiliarySpaceTag, fieldCount = 3.0,
+      **self.argumentsToTemplateConstructors
+    )
+    dim.addRepresentation(threeFieldCoordinateSpace)
+
     fourFieldCoordinateSpace = HermiteGaussDimensionRepresentation(
       name = name + '_4f', type = type, runtimeLattice = lattice, _maximum = maximum,
       stepSizeArray = True, parent = dim, tag = self.auxiliarySpaceTag, fieldCount = 4.0,
       **self.argumentsToTemplateConstructors
     )
     dim.addRepresentation(fourFieldCoordinateSpace)
+
     return dim
   
   def hermiteZeros(self, n):
