@@ -295,7 +295,8 @@ class _TransformMultiplexer (_Feature):
         repNameBasis = tuple([repNameBasis])
       return tuple(representationMap[dimRepName] for dimRepName in repNameBasis)
     
-    vectors = set([v for v in self.getVar('templates') if isinstance(v, VectorElement) and v.needsTransforms])
+    vectors = list([v for v in self.getVar('templates') if isinstance(v, VectorElement) and v.needsTransforms])
+
     
     driver = self._driver
     transformMap = dict()
@@ -346,7 +347,7 @@ class _TransformMultiplexer (_Feature):
     postfixLatticeStringMap = dict()
     
     delayedException = None
-    
+
     for vector in vectors:
       basesNeeded.update(vector.basesNeeded)
       vectorBases = list(vector.basesNeeded)
@@ -378,6 +379,17 @@ class _TransformMultiplexer (_Feature):
           # (think FFT's with different numbers of points not in the FFT dimension)
           geometrySpecification = None
           transformation = self.availableTransformations[transformID]
+          
+          # MTJ: Note that setdefault is a method on a dictionary. 
+          # The setdefault(keyname, value) method returns the value of the item with the specified key.
+          # If the key does not exist, insert the key, with the specified value.
+          # Previously the next bit of code read
+          #   transformation.setdefault('vectors', set()).add(vector)
+          # i.e. if "transformation" has a key called "vectors" that was a set, add "vector"
+          # If it doesn't, add a set called "vectors" and add the element "vector"
+          # However we're trying to transition away from using sets so the generated C code
+          # is deterministic; this means we want to use lists instead. This is TODO
+          #
           transformation.setdefault('vectors', set()).add(vector)
           
           resultBasis, (prefixBasis, matchedSourceBasis, postfixBasis) = transformedBasis(currentBasis, transformPair)
@@ -405,10 +417,10 @@ class _TransformMultiplexer (_Feature):
           postfixLattice = [(dimRep.latticeEstimate, dimRep.localLattice) for dimRep in postfixDimReps if not (dimRep.hasLocalOffset or isinstance(dimRep.runtimeLattice, basestring))]
           postfixLattice.append((vector.nComponents, '_' + vector.id + '_ncomponents'))
           
-          if transformation.get('transformType', 'real') is 'real' and vector.type == 'complex':
+          if transformation.get('transformType', 'real') == 'real' and vector.type == 'complex':
             postfixLattice.append((2, '2'))
           
-          if transformation.get('transformType', 'real') is 'complex' and vector.type == 'real':
+          if transformation.get('transformType', 'real') == 'complex' and vector.type == 'real':
             delayedException = ParserException(
               vector.xmlElement,
               "Vector '%(vectorName)s' is of type 'real', but is needed in the following bases: %(basisList)s.\n"
