@@ -1,26 +1,26 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # encoding: utf-8
 
-import TransformMultiplexer
+from . import TransformMultiplexer
 transformClasses = TransformMultiplexer.TransformMultiplexer.transformClasses
 
-import _NoTransform
-import NoTransformMPI
+from . import _NoTransform
+from . import NoTransformMPI
 _NoTransform._NoTransform.mpiCapableSubclass = NoTransformMPI.NoTransformMPI
 transformClasses['none'] = _NoTransform._NoTransform
 
-import FourierTransformFFTW3
-import FourierTransformFFTW3Threads
-import FourierTransformFFTW3MPI
+from . import FourierTransformFFTW3
+from . import FourierTransformFFTW3Threads
+from . import FourierTransformFFTW3MPI
 FourierTransformFFTW3.FourierTransformFFTW3.mpiCapableSubclass = FourierTransformFFTW3MPI.FourierTransformFFTW3MPI
 transformClasses.update([(name, FourierTransformFFTW3.FourierTransformFFTW3) for name in ['dft', 'dct', 'dst', 'mpi']])
 
-import BesselTransform
+from . import BesselTransform
 transformClasses.update([(name, BesselTransform.BesselTransform) for name in ['bessel', 'spherical-bessel']])
-import BesselNeumannTransform
+from . import BesselNeumannTransform
 transformClasses['bessel-neumann'] = BesselNeumannTransform.BesselNeumannTransform
 
-import HermiteGaussTransform
+from . import HermiteGaussTransform
 transformClasses['hermite-gauss'] = HermiteGaussTransform.HermiteGaussTransform
 
-del transformClasses
\ No newline at end of file
+del transformClasses
