@@ -129,7 +129,13 @@ class _CrossPropagationOperator (Operator):
       reducedVector.type = fullVector.type
       reducedVector.components = fullVector.components
       reducedVector.needsInitialisation = False
-      reducedField.managedVectors.add(reducedVector)
+      
+      # We're trying to convert XMDS's use of sets to lists where
+      # possible to get more repeatable C code. managedVectors is now a
+      # list so can't use the add method
+      # reducedField.managedVectors.add(reducedVector)
+      if reducedVector not in reducedField.managedVectors:
+         reducedField.managedVectors.append(reducedVector)  
     
     self.reducedVectorMap[fullVector] = reducedVector
     self.fullVectorMap[reducedVector] = fullVector
