@@ -59,6 +59,6 @@ def parserWarning(element, msg):
     if (lineNumber, columnNumber, msg) in warningsGiven: return
     warningsGiven.add((lineNumber, columnNumber, msg))
     sys.stderr.write(indentMessageWithPrefix('WARNING: ', msg))
-    sys.stderr.write("    At line %(lineNumber)i, column %(columnNumber)i" % locals())
+    sys.stderr.write("    At line %(lineNumber)i, column %(columnNumber)i\n" % locals())
     # print >> sys.stderr, "    In element: " + element.userUnderstandableXPath()
 
