@@ -31,8 +31,11 @@ class _MultiPathDriver (SimulationDriver):
     super(_MultiPathDriver, self).preflight()
     
     for mg in self.getVar('momentGroups'):
-      mg.processedVector.aliases.add('_%s_sd' % mg.outputField.name)
-    
+      # We need a vector to store the standard error information. Unless
+      # we're exporting all the paths, in which case we're not going to calculate
+      # means or errors.
+      if not mg.export_all_paths:
+        mg.processedVector.aliases.add('_%s_sd' % mg.outputField.name)  
   
   def rawVectorNeedsToBeAllocated(self, dict):
     """
