@@ -42,10 +42,20 @@ class _VectorElement (ScriptElement):
     self.field = field
     if not 'parent' in KWs: KWs['parent'] = field
     
-    if KWs['parent'] is field:
-      field.managedVectors.add(self)
+    if KWs['parent'] is field:    
+      # We're trying to convert XMDS's use of sets to lists where
+      # possible to get more repeatable C code. managedVectors is now a
+      # list so can't use add method
+      # field.managedVectors.add(self)
+      if self not in field.managedVectors:
+        field.managedVectors.append(self)
     else:
-      field.temporaryVectors.add(self)
+      # We're trying to convert XMDS's use of sets to lists where
+      # possible to get more repeatable C code. temporaryVectors
+      # is now a list rather than a set.
+      #field.temporaryVectors.add(self)
+      if self not in field.temporaryVectors:
+        field.temporaryVectors.append(self)
     
     ScriptElement.__init__(self, *args, **KWs)
     
@@ -156,8 +166,24 @@ class _VectorElement (ScriptElement):
     return basis in self.transformMap['bases']
   
   def remove(self):
-    self.field.managedVectors.discard(self)
-    self.field.temporaryVectors.discard(self)
+    # We're trying to convert XMDS's use of sets to lists where
+    # possible to get more repeatable C code. managedVectors is now a
+    # list so can't use discard method
+    # self.field.managedVectors.discard(self)
+    try:
+      self.field.managedVectors.remove(self)
+    except ValueError:
+      pass  # do nothing!
+    
+    # We're trying to convert XMDS's use of sets to lists where
+    # possible to get more repeatable C code. temporaryVectors
+    # is now a list rather than a set.
+    #self.field.temporaryVectors.discard(self)
+    
+    try:
+      self.field.temporaryVectors.remove(self)
+    except ValueError:
+      pass  # do nothing!
     
     super(_VectorElement, self).remove()
   
