@@ -153,10 +153,13 @@ class XSILDataHDF5(XSILData):
     subgroup = f[groupName]
     
     for independentVariable in self.independentVariables:
-      independentVariable['array'] = subgroup[independentVariable['name']].value
-    
+      #independentVariable['array'] = subgroup[independentVariable['name']].value
+      # Fix the warning "H5pyDeprecationWarning: dataset.value has been deprecated. Use dataset[()] instead."
+      independentVariable['array'] = subgroup[independentVariable['name']][()]
     for dependentVariable in self.dependentVariables:
-      dependentVariable['array'] = subgroup[dependentVariable['name']].value
+      #dependentVariable['array'] = subgroup[dependentVariable['name']].value
+      # Fix the warning "H5pyDeprecationWarning: dataset.value has been deprecated. Use dataset[()] instead."
+      dependentVariable['array'] = subgroup[dependentVariable['name']][()]
     
     # Now wasn't that easy.
   
