@@ -82,7 +82,12 @@ class _ScriptElement (Template):
     
     Template.__init__(self, *args, **KWs)
     
-    self.getVar('templates').add(self)
+    # Previously "templates" was a set and used the add method. It's now a list
+    # to try and get more repeatable C code. However sets can't have duplicates
+    # but lists can, so work around that to get same behaviour 
+    temp_templates = self.getVar('templates')
+    if self not in temp_templates:
+      self.getVar('templates').append(self)
     
     # Only set the dependencies attribute if it isn't taken
     # care of elsewhere
@@ -449,7 +454,17 @@ class _ScriptElement (Template):
     Remove the template from various global lists that it may have got itself into. This method
     should be called if a template is no longer needed and should be deleted.
     """
-    self.getVar('templates').discard(self)
+    
+    # Previously "templates" was a set and used the discard method. It's now a list
+    # to try and get repeatable generated C code, so we use the remove method. Only
+    # problem is that unlike discard, remove throw and error if the entry doesn't
+    # exist. Work around this to ensure the result is the same as when we were
+    # using a set instead of a list
+    # self.getVar('templates').discard(self)
+    try:
+      self.getVar('templates').remove(self)
+    except ValueError:
+      pass  # do nothing!
     
     for someIterable in (self.parent.children, self.getVar('fields'),
                          self.getVar('simulationVectors'), self.getVar('momentGroups')):
