@@ -303,7 +303,7 @@ def main(argv=None):
   globalNameSpace['symbolNames'] = set()
   globalNameSpace['xmds'] = {'versionString': versionString,
                              'subversionRevision': subversionRevisionString}
-  globalNameSpace['templates'] = set()
+  globalNameSpace['templates'] = list()
   globalNameSpace['precision'] = 'double'
   globalNameSpace['simulationBuildVariant'] = set()
   globalNameSpace['simulationUselib'] = set()
